/*
 * Ben NanoNote LCD initialisation, based on uboot-xburst and xburst-tools.
 *
 * Copyright (C) 2001-2002 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 * Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef CONFIG_CPU_JZ4730_MINIPC
#include "minipc_claa070vc01.h"
#include "minipc.h"
#else
#include "nanonote_gpm940b0.h"
#include "nanonote.h"
#endif

#include "xburst_types.h"
#include "jzlcd.h"
#include "sdram.h"
#include "board.h"

extern vidinfo_t panel_info;
static unsigned long lcd_base;

static unsigned int get_line_length()
{
	return ALIGN((panel_info.vl_col * NBITS(panel_info.vl_bpix)) / 8, sizeof(u32));
}

static u32 *get_pixel32(unsigned short h, unsigned short v)
{
	return (u32 *) (lcd_base + v * get_line_length()) + h;
}

static u16 *get_pixel16(unsigned short h, unsigned short v)
{
	return (u16 *) (lcd_base + v * get_line_length()) + h;
}

static void test_pixel32(unsigned short h, unsigned short v)
{
	unsigned short v_max = panel_info.vl_row;
	unsigned short h_max = panel_info.vl_col;
	u32 *pix = get_pixel32(h, v);

	*pix = (
		(((255 * (h_max - h)) / (h_max - 1)) << 16) +
		((((255 * h) / (h_max - 1) + (255 * (v_max - v)) / (v_max - 1)) / 2) << 8) +
		((255 * v) / (v_max - 1))
		);
}

static void test_pixel16_565(unsigned short h, unsigned short v)
{
	unsigned short v_max = panel_info.vl_row;
	unsigned short h_max = panel_info.vl_col;
	u16 *pix = get_pixel16(h, v);

	*pix = (
		(((31 * (h_max - h)) / (h_max - 1)) << 11) +
		((((63 * h) / (h_max - 1) + (63 * (v_max - v)) / (v_max - 1)) / 2) << 5) +
		((31 * v) / (v_max - 1))
		);
}

void test_pixel(unsigned short h, unsigned short v)
{
	switch (panel_info.vl_bpix)
	{
		case LCD_COLOR32:
		test_pixel32(h, v);
		break;

		default:
		test_pixel16_565(h, v);
		break;
	}
}

void clear_pixel32(unsigned short h, unsigned short v)
{
	u32 *pix = get_pixel32(h, v);
	*pix = 0;
}

void clear_pixel16(unsigned short h, unsigned short v)
{
	u16 *pix = get_pixel16(h, v);
	*pix = 0;
}

void clear_pixel(unsigned short h, unsigned short v)
{
	switch (panel_info.vl_bpix)
	{
		case LCD_COLOR32:
		clear_pixel32(h, v);
		break;

		default:
		clear_pixel16(h, v);
		break;
	}
}

void test_pattern()
{
	unsigned short v_max  = panel_info.vl_row;
	unsigned short h_max  = panel_info.vl_col;
	unsigned short v, h;

	for (v = 0; v < v_max; v += 1) {
		for (h = 0; h < h_max; h += 1) {
			test_pixel(h, v);
		}
	}
}

void lcd_clear(unsigned long lcd_base)
{
	unsigned short v_max  = panel_info.vl_row;
	unsigned short h_max  = panel_info.vl_col;
	unsigned short v, h;
	unsigned long *pix = (unsigned long *)lcd_base;

	for (v = 0; v < v_max; v += 1) {
		for (h = 0; h < h_max; h += 1) {
			*pix++ = 0;
		}
	}
}

/* LCD initialisation. */

void lcd_init(void)
{
	__lcd_display_pin_init();
	__lcd_display_on();

	lcd_base = lcd_ctrl_init();
	lcd_clear(lcd_base);
	lcd_enable();
}
